@@ -8,8 +8,9 @@
 # @alias toPNG
 # @alias toSVG
 # @alias toTIFF
-# @alias toWMF
+# @alias toWMF 
 # @alias toFavicon
+# @alias toNullDev
 # @alias asDataURI
 #
 # % Interactive/screen devices
@@ -20,6 +21,7 @@
 # @alias toCairoWin
 # @alias toCairoX11
 # %% @alias toJavaGD
+#@alias toRStudioGD
 #
 # @title "Methods for creating image files of a specific format"
 #
@@ -49,11 +51,14 @@
 #   toCairoWin(name, ...)
 #   toCairoX11(name, ...)
 #   %% toJavaGD(name, ...)
+#
+#   % Interactive/screen device for RStudio
+#   toRStudioGD(name, ..., .allowUnknownArgs = TRUE)
 # }
 #
 # \arguments{
 #   \item{name}{A @character string specifying the name of the image file.}
-#   \item{...}{Additional arguments passed to @see "devEval", e.g.
+#   \item{..., .allowUnknownArgs}{Additional arguments passed to @see "devEval", e.g.
 #      \code{tags} and \code{aspectRatio}.}
 #   \item{ext, field}{Passed to @see "devEval".}
 # }
@@ -132,6 +137,10 @@ toFavicon <- function(..., name="favicon", ext="png", field=getDevOption("favico
   }
 } # toFavicon()
 
+toNullDev <- function(name, ...) {
+  devEval(type = "nulldev", name = name, ...)
+}
+
 asDataURI <- function(..., mime=NULL) {
   img <- DevEvalFileProduct(...);
   if (is.null(mime)) {
@@ -165,55 +174,51 @@ toCairoX11 <- function(name, ...) {
   devEval(type="CairoX11", name=name, ...);
 }
 
+toRStudioGD <- function(name, ..., .allowUnknownArgs = TRUE) {
+  ## WORKAROUND: Calling dev.new() in RStudio will open a RStudioGD device
+  ## unless it is already opened, which in case it will open new graphics
+  ## devices as when done outside of RStudio. In other words, there can only
+  ## be one RStudioGD device open at any time.
+  ## Because of the above, call to toRStudioGD() should set current device to
+  ## the RStudioGD one if already open, otherwise create a new one. To open
+  ## an existing device, all we need to do is set 'name' to its label.
+  ## PS. We cannot just close it and have it reopened, because then we'll
+  ##     loose the plot history in RStudio itself.
+
+  ## (a) RStudioGD is not opened
+  idx <- which("RStudioGD" == names(dev.list())) + 1L
+  if (length(idx) == 0) {
+    return(devEval(type = "RStudioGD", name = name, ...,
+                   .allowUnknownArgs = .allowUnknownArgs))
+  }
+
+  ## (b) RStudioGD _is_ opened
+  
+  ## Existing label of the RStudioGD device
+  label <- devGetLabel(idx)
+
+  ## Was the expression passed implicitly via 'name' instead?  If so, then we
+  ## need to make sure to call devEval() with 'name' set to the label of the
+  ## current RStudioGD device.
+  expr <- substitute(name)
+  class <- eval(expression(class(expr)))
+  is_expression <- is.element(class, c("call", "{", "("))
+  if (isTRUE(is_expression)) {
+    args <- list(type = "RStudioGD", name = label, expr = expr, ...,
+                 .allowUnknownArgs = .allowUnknownArgs)
+    return(do.call(devEval, args = args, envir = parent.frame()))
+  }
+
+  ## Otherwise, just call it as is
+  res <- devEval(type = "RStudioGD", name = label, ...,
+                 .allowUnknownArgs = .allowUnknownArgs)
+
+  ## Should the label of the RStudioGD device be updated  
+  if (name != label) devSetLabel(idx, label = name)
+  
+  res
+}
+
 ## toJavaGD <- function(name, ...) {
 ##   devEval(type="JavaGD", name=name, ...);
 ## }
-
-
-
-
-
-##############################################################################
-# HISTORY:
-# 2014-09-15
-# o Added support for toFavicon(..., field="htmlscript").  The device
-#   options are such that the default is to use "htmlscript".
-# o CLEANUP: toNnn():s no longer sets 'field' explitly.
-# 2014-08-29
-# o Changed default argument 'field' for toNNN() from "fullname" to
-#   "pathname" as in getOption("devEval/args/field", "pathname").
-#   This should work for most compilers include LaTeX, although in
-#   later it is preferred to use "fullname".
-# 2014-04-28
-# o Waiting with toCairoPNG() and toJavaGD().
-# o Added toDefault(), toX11(), toWindows(), and toQuartz().
-#   Also added toCairoWin() and toCairoX11() for the 'Cairo' package.
-# 2014-01-11
-# o Added asDataURI().
-# 2014-01-10
-# o Added toFavicon().
-# 2013-07-24
-# o Added toEMF()/toWMF().
-# 2013-02-23
-# o Changed default argument 'field' for toNNN() from "fullname"
-#   to getOption("devEval/args/field", "fullname").
-# 2012-05-01
-# o Renamed Rd to toNNN.
-# 2012-04-27
-# o Renamed Rd to toPNG().
-# 2012-04-07
-# o Added toBMP() and toTIFF().
-# 2011-11-05
-# o Added toSVG().
-# o Moved from private R.rsp.addons package to R.utils.
-# o Dropped default 'width' from toPNG(), toPDF() and toEPS().  No longer
-#   required from R.utils v1.9.1.
-# 2011-10-31
-# o CLEANUP: toPNG() etc now utilizes new 'field' argument of devEval().
-# 2011-09-12
-# o Added toEPS().
-# 2011-06-15
-# o Added argument 'field' to toPNG() and toPDF().
-# 2011-05-23
-# o Added toPNG() and toPDF().
-##############################################################################
